/*!
FullCalendar Time Grid Plugin v7.0.0-rc.0
Docs & License: https://fullcalendar.io/docs/timegrid-view
(c) 2024 Adam Shaw
*/
FullCalendar.TimeGrid = (function (exports, core, internal$1, preact, internal$2) {
    'use strict';

    class AllDaySplitter extends internal$1.Splitter {
        getKeyInfo() {
            return {
                allDay: {},
                timed: {},
            };
        }
        getKeysForDateSpan(dateSpan) {
            if (dateSpan.allDay) {
                return ['allDay'];
            }
            return ['timed'];
        }
        getKeysForEventDef(eventDef) {
            if (!eventDef.allDay) {
                return ['timed'];
            }
            if (internal$1.hasBgRendering(eventDef)) {
                return ['timed', 'allDay'];
            }
            return ['allDay'];
        }
    }

    class DayTimeColsSlicer extends internal$1.Slicer {
        sliceRange(range, dayRanges) {
            let segs = [];
            for (let col = 0; col < dayRanges.length; col += 1) {
                let segRange = internal$1.intersectRanges(range, dayRanges[col]);
                if (segRange) {
                    segs.push({
                        startDate: segRange.start,
                        endDate: segRange.end,
                        isStart: segRange.start.valueOf() === range.start.valueOf(),
                        isEnd: segRange.end.valueOf() === range.end.valueOf(),
                        col,
                    });
                }
            }
            return segs;
        }
    }

    /*
    TODO: more DRY with daygrid?
    can be given null/undefined!
    */
    function organizeSegsByCol(segs, colCnt) {
        let segsByCol = [];
        let i;
        for (i = 0; i < colCnt; i += 1) {
            segsByCol.push([]);
        }
        if (segs) {
            for (i = 0; i < segs.length; i += 1) {
                segsByCol[segs[i].col].push(segs[i]);
            }
        }
        return segsByCol;
    }
    /*
    TODO: more DRY with daygrid?
    can be given null/undefined!
    */
    function splitInteractionByCol(ui, colCnt) {
        let byRow = [];
        if (!ui) {
            for (let i = 0; i < colCnt; i += 1) {
                byRow[i] = null;
            }
        }
        else {
            for (let i = 0; i < colCnt; i += 1) {
                byRow[i] = {
                    affectedInstances: ui.affectedInstances,
                    isEvent: ui.isEvent,
                    segs: [],
                };
            }
            for (let seg of ui.segs) {
                byRow[seg.col].segs.push(seg);
            }
        }
        return byRow;
    }

    // potential nice values for the slot-duration and interval-duration
    // from largest to smallest
    const STOCK_SUB_DURATIONS = [
        { hours: 1 },
        { minutes: 30 },
        { minutes: 15 },
        { seconds: 30 },
        { seconds: 15 },
    ];
    function buildSlatMetas(slotMinTime, slotMaxTime, explicitLabelInterval, slotDuration, dateEnv) {
        let dayStart = new Date(0);
        let slatTime = slotMinTime;
        let slatIterator = internal$1.createDuration(0);
        let labelInterval = explicitLabelInterval || computeLabelInterval(slotDuration);
        let metas = [];
        while (internal$1.asRoughMs(slatTime) < internal$1.asRoughMs(slotMaxTime)) {
            let date = dateEnv.add(dayStart, slatTime);
            let isLabeled = internal$1.wholeDivideDurations(slatIterator, labelInterval) !== null;
            metas.push({
                date,
                time: slatTime,
                key: date.toISOString(),
                isoTimeStr: internal$1.formatIsoTimeString(date),
                isLabeled,
            });
            slatTime = internal$1.addDurations(slatTime, slotDuration);
            slatIterator = internal$1.addDurations(slatIterator, slotDuration);
        }
        return metas;
    }
    // Computes an automatic value for slotLabelInterval
    function computeLabelInterval(slotDuration) {
        let i;
        let labelInterval;
        let slotsPerLabel;
        // find the smallest stock label interval that results in more than one slots-per-label
        for (i = STOCK_SUB_DURATIONS.length - 1; i >= 0; i -= 1) {
            labelInterval = internal$1.createDuration(STOCK_SUB_DURATIONS[i]);
            slotsPerLabel = internal$1.wholeDivideDurations(labelInterval, slotDuration);
            if (slotsPerLabel !== null && slotsPerLabel > 1) {
                return labelInterval;
            }
        }
        return slotDuration; // fall back
    }

    class TimeGridAllDayLabel extends internal$1.BaseComponent {
        constructor() {
            super(...arguments);
            // ref
            this.innerElRef = preact.createRef();
        }
        render() {
            let { props } = this;
            let { options, viewApi } = this.context;
            let renderProps = {
                text: options.allDayText,
                view: viewApi,
            };
            return (preact.createElement(internal$1.ContentContainer, { tag: "div", attrs: {
                    role: 'rowheader',
                }, className: 'fc-timegrid-allday-label fc-timegrid-axis fc-cell fc-content-box', style: {
                    width: props.width,
                }, renderProps: renderProps, generatorName: "allDayContent", customGenerator: options.allDayContent, defaultGenerator: renderAllDayInner, classNameGenerator: options.allDayClassNames, didMount: options.allDayDidMount, willUnmount: options.allDayWillUnmount }, (InnerContent) => (preact.createElement(InnerContent, { tag: "span", className: 'fc-timegrid-allday-label-inner fc-timegrid-axis-inner fc-cell-inner fc-padding-sm', elRef: this.innerElRef }))));
        }
        componentDidMount() {
            const { props } = this;
            const innerEl = this.innerElRef.current; // TODO: make dynamic with useEffect
            // TODO: only attach this if refs props present
            this.disconnectInnerWidth = internal$1.watchWidth(innerEl, (width) => {
                internal$1.setRef(props.innerWidthRef, width);
            });
        }
        componentWillUnmount() {
            this.disconnectInnerWidth();
            internal$1.setRef(this.props.innerWidthRef, null);
        }
    }
    function renderAllDayInner(renderProps) {
        return renderProps.text;
    }

    class TimeGridAllDayLane extends internal$1.DateComponent {
        constructor() {
            super(...arguments);
            this.heightRef = preact.createRef();
            this.handleRootEl = (rootEl) => {
                this.rootEl = rootEl;
                if (rootEl) {
                    this.context.registerInteractiveComponent(this, {
                        el: rootEl,
                    });
                }
                else {
                    this.context.unregisterInteractiveComponent(this);
                }
            };
        }
        render() {
            return (preact.createElement(internal$2.DayGridRow, Object.assign({}, this.props, { isTall: true, rootElRef: this.handleRootEl, heightRef: this.heightRef })));
        }
        queryHit(positionLeft, positionTop, elWidth) {
            const { props, context, heightRef } = this;
            const colCnt = props.cells.length;
            const { col, left, right } = internal$2.computeColFromPosition(positionLeft, elWidth, props.colWidth, colCnt, context.isRtl);
            const cell = props.cells[col];
            const cellStartDate = cell.date;
            const cellEndDate = internal$1.addDays(cellStartDate, 1);
            return {
                dateProfile: props.dateProfile,
                dateSpan: Object.assign({ range: {
                        start: cellStartDate,
                        end: cellEndDate,
                    }, allDay: true }, cell.dateSpanProps),
                getDayEl: () => internal$2.getCellEl(this.rootEl, col),
                rect: {
                    left,
                    right,
                    top: 0,
                    bottom: heightRef.current,
                },
                layer: 0,
            };
        }
    }

    function buildTimeColsModel(dateProfile, dateProfileGenerator) {
        let daySeries = new internal$1.DaySeriesModel(dateProfile.renderRange, dateProfileGenerator);
        return new internal$1.DayTableModel(daySeries, false);
    }
    function buildDayRanges(dayTableModel, dateProfile, dateEnv) {
        let ranges = [];
        for (let date of dayTableModel.headerDates) {
            ranges.push({
                start: dateEnv.add(date, dateProfile.slotMinTime),
                end: dateEnv.add(date, dateProfile.slotMaxTime),
            });
        }
        return ranges;
    }
    function computeSlatHeight(expandRows, slatCnt, slatInnerHeight, scrollerHeight) {
        if (!slatInnerHeight || !scrollerHeight) {
            return [undefined, false];
        }
        const slatMinHeight = slatInnerHeight + 1;
        const slatLiquidHeight = scrollerHeight / slatCnt;
        let slatLiquid;
        let slatHeight;
        if (expandRows && slatLiquidHeight >= slatMinHeight) {
            slatLiquid = true;
            slatHeight = slatLiquidHeight;
        }
        else {
            slatLiquid = false;
            slatHeight = slatMinHeight;
        }
        return [slatHeight, slatLiquid];
    }
    /*
    A `startOfDayDate` must be given for avoiding ambiguity over how to treat midnight.
    */
    function computeDateTopFrac(date, dateProfile, startOfDayDate) {
        if (!startOfDayDate) {
            startOfDayDate = internal$1.startOfDay(date);
        }
        return computeTimeTopFrac(internal$1.createDuration(date.valueOf() - startOfDayDate.valueOf()), dateProfile);
    }
    function computeTimeTopFrac(time, dateProfile) {
        const startMs = internal$1.asRoughMs(dateProfile.slotMinTime);
        const endMs = internal$1.asRoughMs(dateProfile.slotMaxTime);
        let frac = (time.milliseconds - startMs) / (endMs - startMs);
        frac = Math.max(0, frac);
        frac = Math.min(1, frac);
        return frac;
    }
    function getSlatRowClassNames(slatMeta) {
        return [
            'fc-timegrid-slot',
            slatMeta.isLabeled ? '' : 'fc-timegrid-slot-minor',
            'fc-flex-row',
        ];
    }

    function computeFgSegVerticals(segs, dateProfile, colDate, slatCnt, slatHeight, // in pixels
    eventMinHeight, // in pixels
    eventShortHeight) {
        const res = [];
        if (slatHeight != null) {
            const totalHeight = slatHeight * slatCnt;
            for (const seg of segs) {
                const startFrac = computeDateTopFrac(seg.startDate, dateProfile, colDate);
                const endFrac = computeDateTopFrac(seg.endDate, dateProfile, colDate);
                const startCoord = startFrac * totalHeight;
                let endCoord = endFrac * totalHeight;
                let height = endCoord - startCoord;
                if (eventMinHeight != null && height < eventMinHeight) {
                    height = eventMinHeight;
                    endCoord = startCoord + height;
                }
                res.push({
                    start: startCoord,
                    end: endCoord,
                    size: height,
                    isShort: eventShortHeight != null && height < eventShortHeight
                });
            }
        }
        return res;
    }

    /*
    segs assumed sorted
    */
    function buildWebPositioning(segs, segVerticals, strictOrder, maxDepth) {
        const segRanges = [];
        // isn't it true that there will either be ALL hcoords or NONE? can optimize
        for (let i = 0; i < segs.length; i++) {
            const segVertical = segVerticals[i];
            if (segVertical) {
                segRanges.push(Object.assign(Object.assign({}, segs[i]), { start: segVertical.start, end: segVertical.end }));
            }
        }
        const hierarchy = new internal$1.SegHierarchy(segRanges, undefined, // 1 thickness for all segs
        strictOrder, undefined, // maxCoord
        maxDepth);
        let web = buildWeb(hierarchy);
        web = stretchWeb(web, 1); // all levelCoords/thickness will have 0.0-1.0
        const segRects = webToRects(web);
        const hiddenGroups = internal$1.groupIntersectingSegs(hierarchy.hiddenSegs);
        return [segRects, hiddenGroups];
    }
    /*
    TODO: use SegHierarchy::traverseSegs for this?
    */
    function buildWeb(hierarchy) {
        const { placementsByLevel } = hierarchy;
        const buildNode = cacheable((level, lateral) => level + ':' + lateral, (level, lateral) => {
            let siblingRange = findNextLevelSegs(hierarchy, level, lateral);
            let [nextLevelNodes, maxPressure] = buildNodes(siblingRange, buildNode);
            let segPlacement = placementsByLevel[level][lateral];
            return [
                Object.assign(Object.assign({}, segPlacement), { nextLevelNodes }),
                segPlacement.thickness + maxPressure, // the pressure builds
            ];
        });
        const [topLevelNodes] = buildNodes(placementsByLevel.length
            ? { level: 0, lateralStart: 0, lateralEnd: placementsByLevel[0].length }
            : null, buildNode);
        return topLevelNodes;
    }
    function buildNodes(siblingRange, buildNode) {
        if (!siblingRange) {
            return [[], 0];
        }
        let { level, lateralStart, lateralEnd } = siblingRange;
        let lateral = lateralStart;
        let pairs = [];
        while (lateral < lateralEnd) {
            pairs.push(buildNode(level, lateral));
            lateral += 1;
        }
        pairs.sort(cmpDescPressures);
        return [
            pairs.map(extractNode),
            pairs[0][1], // first item's pressure
        ];
    }
    function cmpDescPressures(a, b) {
        return b[1] - a[1];
    }
    function extractNode(a) {
        return a[0];
    }
    function findNextLevelSegs(hierarchy, subjectLevel, subjectLateral) {
        let { levelCoords, placementsByLevel } = hierarchy;
        let subjectPlacement = placementsByLevel[subjectLevel][subjectLateral];
        let afterSubject = levelCoords[subjectLevel] + subjectPlacement.thickness;
        let levelCnt = levelCoords.length;
        let level = subjectLevel;
        // skip past levels that are too high up
        for (; level < levelCnt && levelCoords[level] < afterSubject; level += 1)
            ; // do nothing
        for (; level < levelCnt; level += 1) {
            let placements = placementsByLevel[level];
            let placement;
            let searchIndex = internal$1.binarySearch(placements, subjectPlacement.start, internal$1.getCoordRangeEnd);
            let lateralStart = searchIndex[0] + searchIndex[1]; // if exact match (which doesn't collide), go to next one
            let lateralEnd = lateralStart;
            while ( // loop through placements that horizontally intersect
            (placement = placements[lateralEnd]) && // but not past the whole seg list
                placement.start < subjectPlacement.end) {
                lateralEnd += 1;
            }
            if (lateralStart < lateralEnd) {
                return { level, lateralStart, lateralEnd };
            }
        }
        return null;
    }
    function stretchWeb(topLevelNodes, totalThickness) {
        const stretchNode = cacheable((node, startCoord, prevThickness) => internal$1.getEventKey(node), (node, startCoord, prevThickness) => {
            let { nextLevelNodes, thickness } = node;
            let allThickness = thickness + prevThickness;
            let thicknessFraction = thickness / allThickness;
            let endCoord;
            let newChildren = [];
            if (!nextLevelNodes.length) {
                endCoord = totalThickness;
            }
            else {
                for (let childNode of nextLevelNodes) {
                    if (endCoord === undefined) {
                        let res = stretchNode(childNode, startCoord, allThickness);
                        endCoord = res[0];
                        newChildren.push(res[1]);
                    }
                    else {
                        let res = stretchNode(childNode, endCoord, 0);
                        newChildren.push(res[1]);
                    }
                }
            }
            let newThickness = (endCoord - startCoord) * thicknessFraction;
            return [endCoord - newThickness, Object.assign(Object.assign({}, node), { thickness: newThickness, nextLevelNodes: newChildren })];
        });
        return topLevelNodes.map((node) => stretchNode(node, 0, 0)[1]);
    }
    // not sorted in any particular order
    function webToRects(topLevelNodes) {
        let rectMap = new Map();
        /*
        Returns max stackForward of the node's forward children
        */
        const processNode = cacheable((node, levelCoord, stackDepth) => internal$1.getEventKey(node), (node, levelCoord, stackDepth) => {
            let rect = Object.assign(Object.assign({}, node), { levelCoord,
                stackDepth, stackForward: 0 });
            rectMap.set(rect.eventRange.instance.instanceId, rect);
            return (rect.stackForward = processNodes(node.nextLevelNodes, levelCoord + node.thickness, stackDepth + 1));
        });
        /*
        Returns max stackForward of all `nodes`
        */
        function processNodes(nodes, levelCoord, stackDepth) {
            let stackForward = 0;
            for (let node of nodes) {
                stackForward = Math.max(processNode(node, levelCoord, stackDepth) + 1, stackForward);
            }
            return stackForward;
        }
        processNodes(topLevelNodes, 0, 0);
        return rectMap;
    }
    // TODO: move to general util
    function cacheable(keyFunc, workFunc) {
        const cache = {};
        return (...args) => {
            let key = keyFunc(...args);
            return (key in cache)
                ? cache[key]
                : (cache[key] = workFunc(...args));
        };
    }

    const DEFAULT_TIME_FORMAT = internal$1.createFormatter({
        hour: 'numeric',
        minute: '2-digit',
        meridiem: false,
    });
    class TimeGridEvent extends internal$1.BaseComponent {
        render() {
            const { props } = this;
            return (preact.createElement(internal$1.StandardEvent, Object.assign({}, props, { className: internal$1.joinClassNames('fc-timegrid-event', props.isShort && 'fc-timegrid-event-short', props.isInset && 'fc-timegrid-event-inset', 'fc-v-event', props.isLiquid && 'fc-liquid'), defaultTimeFormat: DEFAULT_TIME_FORMAT })));
        }
    }

    class TimeGridMoreLink extends internal$1.BaseComponent {
        render() {
            let { props } = this;
            return (preact.createElement(internal$1.MoreLinkContainer, { className: 'fc-timegrid-more-link fc-abs', style: {
                    top: props.top,
                    height: props.height,
                }, allDayDate: null, segs: props.hiddenSegs, hiddenSegs: props.hiddenSegs, dateSpanProps: props.dateSpanProps, dateProfile: props.dateProfile, todayRange: props.todayRange, popoverContent: () => renderPlainFgSegs(props.hiddenSegs, props), defaultGenerator: renderMoreLinkInner, forceTimed: true }, (InnerContent) => (preact.createElement(InnerContent, { tag: "div", className: 'fc-timegrid-more-link-inner fc-sticky-t' }))));
        }
    }
    function renderMoreLinkInner(props) {
        return props.shortText;
    }

    function TimeGridNowIndicatorLine(props) {
        return (preact.createElement("div", { className: "fc-timegrid-now-indicator-container" },
            preact.createElement(internal$1.NowIndicatorContainer, { className: 'fc-timegrid-now-indicator-line', style: {
                    top: props.totalHeight != null
                        ? props.totalHeight * computeDateTopFrac(props.nowDate, props.dateProfile, props.dayDate)
                        : undefined
                }, isAxis: false, date: props.nowDate })));
    }

    // Firefox is terrible at rendering absolute elements that span across multiple print pages
    const simplifiedTimeGridPrint = /* true || */ (typeof navigator !== 'undefined' &&
        navigator.userAgent.toLowerCase().includes('firefox'));
    class TimeGridCol extends internal$1.BaseComponent {
        constructor() {
            super(...arguments);
            this.sortEventSegs = internal$1.memoize(internal$1.sortEventSegs);
            this.getDateMeta = internal$1.memoize(internal$1.getDateMeta);
        }
        render() {
            let { props, context } = this;
            let { options } = context;
            let isSelectMirror = options.selectMirror;
            let mirrorSegs = // yuck
             (props.eventDrag && props.eventDrag.segs) ||
                (props.eventResize && props.eventResize.segs) ||
                (isSelectMirror && props.dateSelectionSegs) ||
                [];
            let interactionAffectedInstances = // TODO: messy way to compute this
             (props.eventDrag && props.eventDrag.affectedInstances) ||
                (props.eventResize && props.eventResize.affectedInstances) ||
                {};
            let dateMeta = this.getDateMeta(props.date, props.todayRange, null, props.dateProfile);
            const baseClassName = internal$1.joinClassNames('fc-timegrid-day', props.borderStart && 'fc-border-s', props.width == null && 'fc-liquid', 'fc-flex-col fc-rel');
            const baseStyle = {
                width: props.width,
                zIndex: 1, // get above slots
            };
            if (dateMeta.isDisabled) {
                return (preact.createElement("div", { role: 'gridcell', "aria-disabled": true, className: internal$1.joinClassNames(baseClassName, 'fc-day-disabled'), style: baseStyle }));
            }
            let sortedFgSegs = this.sortEventSegs(props.fgEventSegs, options.eventOrder);
            return (preact.createElement(internal$1.DayCellContainer, { tag: "div", attrs: Object.assign(Object.assign({}, props.attrs), { role: 'gridcell' }), className: baseClassName, style: baseStyle, date: props.date, dateMeta: dateMeta, renderProps: props.renderProps }, (InnerContent) => (preact.createElement(preact.Fragment, null,
                this.renderFillSegs(props.businessHourSegs, 'non-business'),
                this.renderFillSegs(props.bgEventSegs, 'bg-event'),
                this.renderFillSegs(props.dateSelectionSegs, 'highlight'),
                internal$1.hasCustomDayCellContent(options) && (preact.createElement(InnerContent, { tag: "div", className: 'fc-timegrid-day-misc fc-fill-top' })),
                preact.createElement("div", { className: internal$1.joinClassNames('fc-timegrid-day-events', (props.forPrint && simplifiedTimeGridPrint)
                        ? 'fc-timegrid-day-events-simple'
                        : 'fc-fill') }, this.renderFgSegs(sortedFgSegs, interactionAffectedInstances, false, false, false)),
                Boolean(mirrorSegs.length) && (
                // has natural z-index to be above other fg container,
                // but only show it when there are actual mirror events, to avoid blocking clicks
                preact.createElement("div", { className: 'fc-timegrid-day-events fc-fill' }, this.renderFgSegs(mirrorSegs, {}, Boolean(props.eventDrag), Boolean(props.eventResize), Boolean(isSelectMirror), 'mirror'))),
                this.renderNowIndicator(props.nowIndicatorSegs)))));
        }
        renderFgSegs(sortedFgSegs, segIsInvisible, isDragging, isResizing, isDateSelecting, forcedKey) {
            const { props } = this;
            if (props.forPrint && simplifiedTimeGridPrint) {
                return renderPlainFgSegs(sortedFgSegs, props);
            }
            return this.renderPositionedFgSegs(sortedFgSegs, segIsInvisible, isDragging, isResizing, isDateSelecting, forcedKey);
        }
        renderPositionedFgSegs(segs, // if not mirror, needs to be sorted
        segIsInvisible, isDragging, isResizing, isDateSelecting, forcedKey) {
            let { props, context } = this;
            let { date, dateProfile, eventSelection, todayRange, nowDate } = props;
            let { eventMaxStack, eventShortHeight, eventOrderStrict, eventMinHeight } = context.options;
            // TODO: memoize this?
            let segVerticals = computeFgSegVerticals(segs, dateProfile, date, props.slatCnt, props.slatHeight, eventMinHeight, eventShortHeight);
            let [segRects, hiddenGroups] = buildWebPositioning(segs, segVerticals, eventOrderStrict, eventMaxStack);
            let isMirror = isDragging || isResizing || isDateSelecting;
            return (preact.createElement(preact.Fragment, null,
                segs.map((seg, index) => {
                    let { eventRange } = seg;
                    let instanceId = eventRange.instance.instanceId; // guaranteed because it's an fg event
                    let segVertical = segVerticals[index] || {};
                    let segRect = segRects.get(instanceId); // for horizontals. could be undefined!? HACK
                    let hStyle = (!isMirror && segRect)
                        ? this.computeSegHStyle(segRect)
                        : { left: 0, right: 0 };
                    let isVisible = isMirror || (segRect && !segIsInvisible[instanceId]);
                    let isInset = segRect ? Boolean(segRect.stackDepth) : false;
                    return (preact.createElement("div", { 
                        // we would have used fc-fill, but multi-page spanning breaks in Firefox
                        // we would have used height:100%, but multi-page spanning breaks in Safari
                        className: 'fc-abs fc-flex-col', key: forcedKey || instanceId, style: Object.assign({ visibility: isVisible ? '' : 'hidden', top: segVertical.start, height: segVertical.size }, hStyle) },
                        preact.createElement(TimeGridEvent, Object.assign({ eventRange: eventRange, slicedStart: seg.startDate, slicedEnd: seg.endDate, isStart: seg.isStart, isEnd: seg.isEnd, isDragging: isDragging, isResizing: isResizing, isDateSelecting: isDateSelecting, isSelected: instanceId === eventSelection, isShort: segVertical.isShort || false, isInset: isInset, isLiquid: true }, internal$1.getEventRangeMeta(eventRange, todayRange, nowDate)))));
                }),
                this.renderHiddenGroups(hiddenGroups)));
        }
        /*
        NOTE: will already have eventMinHeight applied because segEntries(?) already had it
        */
        renderHiddenGroups(hiddenGroups) {
            let { dateSpanProps, dateProfile, todayRange, nowDate, eventSelection, eventDrag, eventResize } = this.props;
            return (preact.createElement(preact.Fragment, null, hiddenGroups.map((hiddenGroup) => {
                return (preact.createElement(TimeGridMoreLink, { key: hiddenGroup.key, hiddenSegs: hiddenGroup.segs, top: hiddenGroup.start, height: hiddenGroup.end - hiddenGroup.start, dateSpanProps: dateSpanProps, dateProfile: dateProfile, todayRange: todayRange, nowDate: nowDate, eventSelection: eventSelection, eventDrag: eventDrag, eventResize: eventResize }));
            })));
        }
        renderFillSegs(segs, fillType) {
            let { props, context } = this;
            let segVerticals = computeFgSegVerticals(segs, props.dateProfile, props.date, props.slatCnt, props.slatHeight, context.options.eventMinHeight);
            return (preact.createElement(preact.Fragment, null, segs.map((seg, index) => {
                const { eventRange } = seg;
                const segVertical = segVerticals[index] || {};
                return (preact.createElement("div", { key: internal$1.buildEventRangeKey(eventRange), className: "fc-fill-x", style: {
                        top: segVertical.start,
                        height: segVertical.size,
                    } }, fillType === 'bg-event' ?
                    preact.createElement(internal$1.BgEvent, Object.assign({ eventRange: eventRange, isStart: seg.isStart, isEnd: seg.isEnd }, internal$1.getEventRangeMeta(eventRange, props.todayRange, props.nowDate))) :
                    internal$1.renderFill(fillType)));
            })));
        }
        renderNowIndicator(segs) {
            let { props } = this;
            if (props.forPrint && simplifiedTimeGridPrint) {
                return;
            }
            // TODO: what if nowIndicator turned OFF??
            return segs.map((seg) => (preact.createElement(TimeGridNowIndicatorLine, { nowDate: seg.startDate, dayDate: props.date, dateProfile: props.dateProfile, totalHeight: props.slatHeight != null ? props.slatHeight * props.slatCnt : undefined })));
        }
        /*
        TODO: eventually move to width, not left+right
        */
        computeSegHStyle(segRect) {
            let { isRtl, options } = this.context;
            let shouldOverlap = options.slotEventOverlap;
            let nearCoord = segRect.levelCoord; // the left side if LTR. the right side if RTL. floating-point
            let farCoord = segRect.levelCoord + segRect.thickness; // the right side if LTR. the left side if RTL. floating-point
            let left; // amount of space from left edge, a fraction of the total width
            let right; // amount of space from right edge, a fraction of the total width
            if (shouldOverlap) {
                // double the width, but don't go beyond the maximum forward coordinate (1.0)
                farCoord = Math.min(1, nearCoord + (farCoord - nearCoord) * 2);
            }
            if (isRtl) {
                left = 1 - farCoord;
                right = nearCoord;
            }
            else {
                left = nearCoord;
                right = 1 - farCoord;
            }
            let props = {
                zIndex: segRect.stackDepth + 1,
                left: internal$1.fracToCssDim(left),
                right: internal$1.fracToCssDim(right),
            };
            if (shouldOverlap && segRect.stackForward) {
                // add padding to the edge so that forward stacked events don't cover the resizer's icon
                props[isRtl ? 'marginLeft' : 'marginRight'] = 10 * 2; // 10 is a guesstimate of the icon's width
            }
            return props;
        }
    }
    function renderPlainFgSegs(sortedFgSegs, { todayRange, nowDate, eventSelection, eventDrag, eventResize }) {
        let hiddenInstances = (eventDrag ? eventDrag.affectedInstances : null) ||
            (eventResize ? eventResize.affectedInstances : null) ||
            {};
        return (preact.createElement(preact.Fragment, null, sortedFgSegs.map((seg) => {
            let { eventRange } = seg;
            let { instanceId } = eventRange.instance;
            return (preact.createElement("div", { key: instanceId, className: 'fc-break-inside-avoid', style: { visibility: hiddenInstances[instanceId] ? 'hidden' : '' } },
                preact.createElement(TimeGridEvent, Object.assign({ eventRange: eventRange, slicedStart: seg.startDate, slicedEnd: seg.endDate, isStart: seg.isStart, isEnd: seg.isEnd, isDragging: false, isResizing: false, isDateSelecting: false, isSelected: instanceId === eventSelection, isShort: false, isInset: false }, internal$1.getEventRangeMeta(eventRange, todayRange, nowDate)))));
        })));
    }

    class TimeGridCols extends internal$1.DateComponent {
        constructor() {
            super(...arguments);
            // memo
            this.processSlotOptions = internal$1.memoize(processSlotOptions);
            this.handleRootEl = (el) => {
                this.rootEl = el;
                if (el) {
                    this.context.registerInteractiveComponent(this, {
                        el,
                        isHitComboAllowed: this.props.isHitComboAllowed,
                    });
                }
                else {
                    this.context.unregisterInteractiveComponent(this);
                }
            };
        }
        render() {
            const { props } = this;
            return (preact.createElement("div", { role: props.role /* !!! */, className: internal$1.joinClassNames(props.className, 'fc-flex-row'), ref: this.handleRootEl }, props.cells.map((cell, col) => (preact.createElement(TimeGridCol, { key: cell.key, dateProfile: props.dateProfile, nowDate: props.nowDate, todayRange: props.todayRange, date: cell.date, slatCnt: props.slatCnt, renderProps: cell.renderProps, attrs: cell.attrs, dateSpanProps: cell.dateSpanProps, forPrint: props.forPrint, borderStart: Boolean(col), 
                // content
                fgEventSegs: props.fgEventSegsByCol[col], bgEventSegs: props.bgEventSegsByCol[col], businessHourSegs: props.businessHourSegsByCol[col], nowIndicatorSegs: props.nowIndicatorSegsByCol[col], dateSelectionSegs: props.dateSelectionSegsByCol[col], eventDrag: props.eventDragByCol[col], eventResize: props.eventResizeByCol[col], eventSelection: props.eventSelection, 
                // dimensions
                width: props.colWidth, slatHeight: props.slatHeight })))));
        }
        queryHit(positionLeft, positionTop, elWidth) {
            const { dateProfile, cells, colWidth, slatHeight } = this.props;
            const { dateEnv, options, isRtl } = this.context;
            const { snapDuration, snapsPerSlot } = this.processSlotOptions(options.slotDuration, options.snapDuration);
            const colCnt = cells.length;
            const { col, left, right } = internal$2.computeColFromPosition(positionLeft, elWidth, colWidth, colCnt, isRtl);
            const cell = cells[col];
            const slatIndex = Math.floor(positionTop / slatHeight);
            const slatTop = slatIndex * slatHeight;
            const partial = (positionTop - slatTop) / slatHeight; // floating point number between 0 and 1
            const localSnapIndex = Math.floor(partial * snapsPerSlot); // the snap # relative to start of slat
            const snapIndex = slatIndex + localSnapIndex * snapsPerSlot;
            const time = internal$1.addDurations(dateProfile.slotMinTime, internal$1.multiplyDuration(snapDuration, snapIndex));
            const start = dateEnv.add(cell.date, time);
            const end = dateEnv.add(start, snapDuration);
            return {
                dateProfile,
                dateSpan: Object.assign({ range: { start, end }, allDay: false }, cell.dateSpanProps),
                getDayEl: () => internal$2.getCellEl(this.rootEl, col),
                rect: {
                    left,
                    right,
                    top: slatTop,
                    bottom: slatTop + slatHeight,
                },
                layer: 0,
            };
        }
    }
    TimeGridCols.addPropsEquality({
        style: internal$1.isPropsEqual,
    });
    // Utils
    // -------------------------------------------------------------------------------------------------
    function processSlotOptions(slotDuration, snapDurationOverride) {
        let snapDuration = snapDurationOverride || slotDuration;
        let snapsPerSlot = internal$1.wholeDivideDurations(slotDuration, snapDuration);
        if (snapsPerSlot === null) {
            snapDuration = slotDuration;
            snapsPerSlot = 1;
            // TODO: say warning?
        }
        return { snapDuration, snapsPerSlot };
    }

    function TimeGridNowIndicatorArrow(props) {
        return (preact.createElement("div", { className: "fc-timegrid-now-indicator-container" },
            preact.createElement(internal$1.NowIndicatorContainer, { className: 'fc-timegrid-now-indicator-arrow', style: {
                    top: props.totalHeight != null
                        ? props.totalHeight * computeDateTopFrac(props.nowDate, props.dateProfile)
                        : undefined
                }, isAxis: true, date: props.nowDate })));
    }

    const DEFAULT_SLAT_LABEL_FORMAT = internal$1.createFormatter({
        hour: 'numeric',
        minute: '2-digit',
        omitZeroMinute: true,
        meridiem: 'short',
    });
    function createRenderProps(date, time, labelFormat, dateEnv, viewApi) {
        return {
            level: 0,
            time: time,
            date: dateEnv.toDate(date),
            view: viewApi,
            text: dateEnv.format(date, labelFormat),
        };
    }
    class TimeGridSlatLabel extends internal$1.BaseComponent {
        constructor() {
            super(...arguments);
            // memo
            this.createRenderProps = internal$1.memoize(createRenderProps);
            // ref
            this.innerElRef = preact.createRef();
        }
        render() {
            let { props, context } = this;
            let { dateEnv, options, viewApi } = context;
            let className = internal$1.joinClassNames('fc-timegrid-slot-label fc-timegrid-axis fc-header-cell fc-cell', props.isLiquid ? 'fc-liquid' : 'fc-content-box');
            if (!props.isLabeled) {
                return (preact.createElement("div", { className: className, style: { width: props.width } }));
            }
            let labelFormat = // TODO: fully pre-parse
             options.slotLabelFormat == null ? DEFAULT_SLAT_LABEL_FORMAT :
                Array.isArray(options.slotLabelFormat) ? internal$1.createFormatter(options.slotLabelFormat[0]) :
                    internal$1.createFormatter(options.slotLabelFormat);
            let renderProps = this.createRenderProps(props.date, props.time, labelFormat, dateEnv, viewApi);
            return (preact.createElement(internal$1.ContentContainer, { tag: "div", className: className, style: { width: props.width }, renderProps: renderProps, generatorName: "slotLabelContent", customGenerator: options.slotLabelContent, defaultGenerator: renderInnerContent, classNameGenerator: options.slotLabelClassNames, didMount: options.slotLabelDidMount, willUnmount: options.slotLabelWillUnmount }, (InnerContent) => (preact.createElement(InnerContent, { tag: "div", className: 'fc-timegrid-axis-inner fc-cell-inner fc-padding-sm', elRef: this.innerElRef }))));
        }
        componentDidMount() {
            const { props } = this;
            const innerEl = this.innerElRef.current; // TODO: make dynamic with useEffect
            if (innerEl) { // could be null if !isLabeled
                // TODO: only attach this if refs props present
                // TODO: fire width/height independently?
                this.disconnectInnerSize = internal$1.watchSize(innerEl, (width, height) => {
                    internal$1.setRef(props.innerWidthRef, width);
                    internal$1.setRef(props.innerHeightRef, height);
                });
            }
        }
        componentWillUnmount() {
            const { props } = this;
            if (this.disconnectInnerSize) {
                this.disconnectInnerSize();
                internal$1.setRef(props.innerWidthRef, null);
                internal$1.setRef(props.innerHeightRef, null);
            }
        }
    }
    function renderInnerContent(props) {
        return props.text;
    }

    class TimeGridSlatLane extends internal$1.BaseComponent {
        constructor() {
            super(...arguments);
            // ref
            this.innerElRef = preact.createRef();
        }
        render() {
            let { props, context } = this;
            let { options } = context;
            let renderProps = {
                time: props.time,
                date: context.dateEnv.toDate(props.date),
                view: context.viewApi,
            };
            return (preact.createElement(internal$1.ContentContainer, { tag: "div", 
                // TODO: have lane classNames like 'fc-timegrid-lane'/'fc-timegrid-slot-lane'/'fc-timegrid-slat-lane'
                className: internal$1.joinClassNames('fc-timegrid-slot-lane fc-cell fc-liquid', props.borderStart && 'fc-border-s'), renderProps: renderProps, generatorName: "slotLaneContent", customGenerator: options.slotLaneContent, classNameGenerator: options.slotLaneClassNames, didMount: options.slotLaneDidMount, willUnmount: options.slotLaneWillUnmount }, (InnerContent) => (preact.createElement(InnerContent, { tag: "div", className: 'fc-cell-inner', elRef: this.innerElRef }))));
        }
        componentDidMount() {
            const innerEl = this.innerElRef.current; // TODO: make dynamic with useEffect
            // TODO: only attach this if refs props present
            this.disconnectInnerHeight = internal$1.watchHeight(innerEl, (height) => {
                internal$1.setRef(this.props.innerHeightRef, height);
            });
        }
        componentWillUnmount() {
            this.disconnectInnerHeight();
            internal$1.setRef(this.props.innerHeightRef, null);
        }
    }

    const DEFAULT_WEEK_NUM_FORMAT = internal$1.createFormatter({ week: 'short' });
    class TimeGridWeekNumber extends internal$1.BaseComponent {
        constructor() {
            super(...arguments);
            // ref
            this.innerElRef = preact.createRef();
        }
        render() {
            let { props, context } = this;
            let range = props.dateProfile.renderRange;
            let dayCnt = internal$1.diffDays(range.start, range.end);
            // HACK: only make week-number a nav-link when NOT in week-view
            let isNavLink = dayCnt === 1 && context.options.navLinks;
            let fullDateStr = internal$1.buildDateStr(context, range.start, 'week');
            return (preact.createElement(internal$1.WeekNumberContainer, { tag: 'div', attrs: {
                    role: 'gridcell',
                    'aria-label': fullDateStr,
                }, className: internal$1.joinClassNames('fc-timegrid-weeknumber fc-timegrid-axis fc-cell', props.isLiquid ? 'fc-liquid' : 'fc-content-box'), style: { width: props.width }, date: range.start, defaultFormat: DEFAULT_WEEK_NUM_FORMAT }, (InnerContent) => (preact.createElement(InnerContent, { tag: 'div', attrs: isNavLink
                    ? internal$1.buildNavLinkAttrs(context, range.start, 'week', fullDateStr)
                    : { 'aria-label': fullDateStr }, className: 'fc-timegrid-axis-inner fc-cell-inner fc-padding-sm', elRef: this.innerElRef }))));
        }
        componentDidMount() {
            const { props } = this;
            const innerEl = this.innerElRef.current; // TODO: make dynamic with useEffect
            // TODO: only attach this if refs props present
            // TODO: handle width/height independently?
            this.disconnectInnerSize = internal$1.watchSize(innerEl, (width, height) => {
                internal$1.setRef(props.innerWidthRef, width);
                internal$1.setRef(props.innerHeightRef, height);
            });
        }
        componentWillUnmount() {
            const { props } = this;
            this.disconnectInnerSize();
            internal$1.setRef(props.innerWidthRef, null);
            internal$1.setRef(props.innerHeightRef, null);
        }
    }

    function TimeGridAxisEmpty(props) {
        return (preact.createElement("div", { role: 'gridcell' // is empty so can't be rowheader/columnheader
            , className: internal$1.joinClassNames('fc-timegrid-axis', props.isLiquid ? 'fc-liquid' : 'fc-content-box'), style: { width: props.width } }));
    }

    class TimeGridLayoutPannable extends internal$1.BaseComponent {
        constructor() {
            super(...arguments);
            this.state = {
                headerTierHeights: [],
            };
            // refs
            this.headerLabelInnerWidthRefMap = new internal$1.RefMap(() => {
                internal$1.afterSize(this.handleAxisWidths);
            });
            this.headerLabelInnerHeightRefMap = new internal$1.RefMap(() => {
                internal$1.afterSize(this.handleHeaderHeights);
            });
            this.headerMainInnerHeightRefMap = new internal$1.RefMap(() => {
                internal$1.afterSize(this.handleHeaderHeights);
            });
            this.handleAllDayLabelInnerWidth = (width) => {
                this.allDayLabelInnerWidth = width;
                internal$1.afterSize(this.handleAxisWidths);
            };
            this.slatLabelInnerWidthRefMap = new internal$1.RefMap(() => {
                internal$1.afterSize(this.handleAxisWidths);
            });
            this.slatLabelInnerHeightRefMap = new internal$1.RefMap(() => {
                internal$1.afterSize(this.handleSlatInnerHeights);
            });
            this.slatMainInnerHeightRefMap = new internal$1.RefMap(() => {
                internal$1.afterSize(this.handleSlatInnerHeights);
            });
            this.headerScrollerRef = preact.createRef();
            this.allDayScrollerRef = preact.createRef();
            this.mainScrollerRef = preact.createRef();
            this.footScrollerRef = preact.createRef();
            this.axisScrollerRef = preact.createRef();
            // Sizing
            // -----------------------------------------------------------------------------------------------
            this.handleTotalWidth = (totalWidth) => {
                this.setState({ totalWidth });
            };
            this.handleClientWidth = (clientWidth) => {
                this.setState({ clientWidth });
            };
            this.handleClientHeight = (clientHeight) => {
                this.setState({ clientHeight });
            };
            this.handleBottomScrollbarWidth = (bottomScrollbarWidth) => {
                this.setState({ bottomScrollbarWidth });
            };
            this.handleHeaderHeights = () => {
                const headerLabelInnerHeightMap = this.headerLabelInnerHeightRefMap.current;
                const headerMainInnerHeightMap = this.headerMainInnerHeightRefMap.current;
                const heights = [];
                // important to loop using 'main' because 'label' might not be tracking height if empty
                for (const [tierNum, mainHeight] of headerMainInnerHeightMap.entries()) {
                    heights[tierNum] = Math.max(headerLabelInnerHeightMap.get(tierNum) || 0, mainHeight);
                }
                const { headerTierHeights } = this.state;
                if (!internal$1.isArraysEqual(headerTierHeights, heights)) {
                    this.setState({ headerTierHeights: heights });
                }
            };
            this.handleSlatInnerHeights = () => {
                const slatLabelInnerHeightMap = this.slatLabelInnerHeightRefMap.current;
                const slatMainInnerHeightMap = this.slatMainInnerHeightRefMap.current;
                let max = 0;
                for (const slatLabelInnerHeight of slatLabelInnerHeightMap.values()) {
                    max = Math.max(max, slatLabelInnerHeight);
                }
                for (const slatMainInnerHeight of slatMainInnerHeightMap.values()) {
                    max = Math.max(max, slatMainInnerHeight);
                }
                if (this.state.slatInnerHeight !== max) {
                    this.setState({ slatInnerHeight: max });
                }
            };
            this.handleAxisWidths = () => {
                const headerLabelInnerWidthMap = this.headerLabelInnerWidthRefMap.current;
                const slatLabelInnerWidthMap = this.slatLabelInnerWidthRefMap.current;
                let max = this.allDayLabelInnerWidth || 0; // guard against all-day slot hidden
                for (const headerLabelInnerWidth of headerLabelInnerWidthMap.values()) {
                    max = Math.max(max, headerLabelInnerWidth);
                }
                for (const slatLableInnerWidth of slatLabelInnerWidthMap.values()) {
                    max = Math.max(max, slatLableInnerWidth);
                }
                if (this.state.axisWidth !== max) {
                    this.setState({ axisWidth: max });
                }
            };
        }
        render() {
            const { props, state, context, headerLabelInnerWidthRefMap, headerLabelInnerHeightRefMap, headerMainInnerHeightRefMap, slatLabelInnerWidthRefMap, slatLabelInnerHeightRefMap, slatMainInnerHeightRefMap, } = this;
            const { nowDate, headerTiers, forPrint } = props;
            const { axisWidth, totalWidth, clientWidth, clientHeight, bottomScrollbarWidth } = state;
            const { options } = context;
            const endScrollbarWidth = (totalWidth != null && clientWidth != null)
                ? totalWidth - clientWidth
                : undefined;
            const verticalScrolling = !forPrint && !internal$1.getIsHeightAuto(options);
            const stickyHeaderDates = !forPrint && internal$1.getStickyHeaderDates(options);
            const stickyFooterScrollbar = !forPrint && internal$1.getStickyFooterScrollbar(options);
            const absPrint = forPrint && !simplifiedTimeGridPrint;
            const simplePrint = forPrint && simplifiedTimeGridPrint;
            const colCnt = props.cells.length;
            // TODO: memo?
            const [canvasWidth, colWidth] = internal$2.computeColWidth(colCnt, props.dayMinWidth, clientWidth);
            const slatCnt = props.slatMetas.length;
            const [slatHeight, slatLiquid] = computeSlatHeight(// TODO: memo?
            verticalScrolling && options.expandRows, slatCnt, state.slatInnerHeight, clientHeight);
            this.slatHeight = slatHeight;
            // TODO: have computeSlatHeight return?
            const totalSlatHeight = (slatHeight || 0) * slatCnt;
            const forcedBodyHeight = absPrint ? totalSlatHeight : undefined;
            const rowsNotExpanding = verticalScrolling && !options.expandRows &&
                clientHeight != null && clientHeight > totalSlatHeight;
            const firstBodyRowIndex = options.dayHeaders ? headerTiers.length + 1 : 1;
            return (preact.createElement(preact.Fragment, null,
                options.dayHeaders && (preact.createElement("div", { className: internal$1.joinClassNames(
                    // see note in TimeGridLayout about why we don't do fc-print-header
                    'fc-timegrid-header fc-flex-row fc-border-b', stickyHeaderDates && 'fc-table-header-sticky') },
                    preact.createElement("div", { role: 'rowgroup', className: 'fc-content-box', style: { width: axisWidth } }, headerTiers.map((rowConfig, tierNum) => (preact.createElement("div", { key: tierNum, role: 'row', "aria-rowindex": tierNum + 1, className: internal$1.joinClassNames('fc-flex-row fc-content-box', tierNum && 'fc-border-t'), style: { height: state.headerTierHeights[tierNum] } }, (options.weekNumbers && rowConfig.isDateRow) ? (preact.createElement(TimeGridWeekNumber, { dateProfile: props.dateProfile, innerWidthRef: headerLabelInnerWidthRefMap.createRef(tierNum), innerHeightRef: headerLabelInnerHeightRefMap.createRef(tierNum), width: undefined, isLiquid: true })) : (preact.createElement(TimeGridAxisEmpty, { width: undefined, isLiquid: true })))))),
                    preact.createElement(internal$1.Scroller, { horizontal: true, hideScrollbars: true, className: 'fc-flex-row fc-border-s fc-liquid', ref: this.headerScrollerRef },
                        preact.createElement("div", { role: 'rowgroup', className: canvasWidth == null ? 'fc-liquid' : '', style: { width: canvasWidth } }, props.headerTiers.map((rowConfig, tierNum) => (preact.createElement(internal$2.DayGridHeaderRow, Object.assign({}, rowConfig, { key: tierNum, role: 'row', rowIndex: tierNum, className: tierNum ? 'fc-border-t' : '', height: state.headerTierHeights[tierNum], colWidth: colWidth, innerHeightRef: headerMainInnerHeightRefMap.createRef(tierNum) }))))),
                        Boolean(endScrollbarWidth) && (preact.createElement("div", { className: 'fc-border-s fc-filler', style: { minWidth: endScrollbarWidth } }))))),
                preact.createElement("div", { role: 'rowgroup', className: internal$1.joinClassNames('fc-flex-col', verticalScrolling && 'fc-liquid') },
                    options.allDaySlot && (preact.createElement(preact.Fragment, null,
                        preact.createElement("div", { role: 'row', "aria-rowindex": firstBodyRowIndex, className: 'fc-timegrid-allday fc-flex-row' },
                            preact.createElement(TimeGridAllDayLabel, { width: axisWidth, innerWidthRef: this.handleAllDayLabelInnerWidth }),
                            preact.createElement(internal$1.Scroller, { horizontal: true, hideScrollbars: true, className: 'fc-border-s fc-flex-row fc-liquid' // fill remaining width
                                , ref: this.allDayScrollerRef },
                                preact.createElement("div", { className: 'fc-flex-col', style: { width: canvasWidth } },
                                    preact.createElement(TimeGridAllDayLane, { dateProfile: props.dateProfile, todayRange: props.todayRange, cells: props.cells, showDayNumbers: false, forPrint: forPrint, isHitComboAllowed: props.isHitComboAllowed, isCompact: internal$2.computeRowIsCompact(clientWidth, options), 
                                        // content
                                        fgEventSegs: props.fgEventSegs, bgEventSegs: props.bgEventSegs, businessHourSegs: props.businessHourSegs, dateSelectionSegs: props.dateSelectionSegs, eventSelection: props.eventSelection, eventDrag: props.eventDrag, eventResize: props.eventResize, dayMaxEvents: props.dayMaxEvents, dayMaxEventRows: props.dayMaxEventRows, 
                                        // dimensions
                                        colWidth: colWidth })),
                                Boolean(endScrollbarWidth) && (preact.createElement("div", { className: 'fc-border-s fc-filler', style: { minWidth: endScrollbarWidth } })))),
                        preact.createElement("div", { className: 'fc-rowdivider' }))),
                    preact.createElement("div", { role: 'row', "aria-rowindex": firstBodyRowIndex + (options.allDaySlot ? 1 : 0), className: internal$1.joinClassNames('fc-timegrid-body fc-flex-row fc-rel', // TODO: rename away from "body"? (because we have another "body" div above)
                        // fc-rel for Ruler.fc-fill-start
                        verticalScrolling && 'fc-liquid') },
                        preact.createElement(internal$1.Scroller, { vertical: verticalScrolling, hideScrollbars: true, className: 'fc-flex-col fc-content-box', style: {
                                width: axisWidth,
                            }, ref: this.axisScrollerRef }, !simplePrint && (preact.createElement(preact.Fragment, null,
                            preact.createElement("div", { role: 'rowheader', "aria-label": options.timedText, 
                                // fc-rel is for absPrint AND TimeGridNowIndicatorArrow
                                className: 'fc-flex-col fc-grow fc-rel', style: {
                                    height: forcedBodyHeight,
                                } },
                                preact.createElement("div", { "aria-hidden": true, className: internal$1.joinClassNames('fc-timegrid-slots-axis fc-flex-col', (verticalScrolling && options.expandRows) && 'fc-grow', absPrint && 'fc-fill-x') }, props.slatMetas.map((slatMeta, slatI) => (preact.createElement("div", { key: slatMeta.key, "data-time": slatMeta.isoTimeStr, className: internal$1.joinClassNames(...getSlatRowClassNames(slatMeta), slatI && 'fc-border-t', slatLiquid && 'fc-liquid'), style: {
                                        height: slatLiquid ? '' : slatHeight
                                    } },
                                    preact.createElement(TimeGridSlatLabel, Object.assign({}, slatMeta /* FYI doesn't need isoTimeStr */, { isLiquid: true, innerWidthRef: slatLabelInnerWidthRefMap.createRef(slatMeta.key), innerHeightRef: slatLabelInnerHeightRefMap.createRef(slatMeta.key) })))))),
                                options.nowIndicator && internal$1.rangeContainsMarker(props.dateProfile.currentRange, nowDate) && (preact.createElement(TimeGridNowIndicatorArrow, { nowDate: nowDate, dateProfile: props.dateProfile, totalHeight: slatHeight != null ? slatHeight * slatCnt : undefined })),
                                Boolean(rowsNotExpanding || bottomScrollbarWidth) && (preact.createElement("div", { class: internal$1.joinClassNames('fc-border-t fc-filler', rowsNotExpanding && 'fc-liquid'), style: {
                                        minHeight: bottomScrollbarWidth
                                    } })))))),
                        preact.createElement("div", { 
                            // we need this div because it's bad for Scroller to have left/right borders,
                            // AND because we need to containt the FooterScrollbar
                            className: 'fc-border-s fc-flex-col fc-liquid' },
                            preact.createElement(internal$1.Scroller, { vertical: verticalScrolling, horizontal: true, hideScrollbars: stickyFooterScrollbar || // also means height:auto, so won't need vertical scrollbars anyway
                                    forPrint, className: internal$1.joinClassNames('fc-flex-col fc-rel', // fc-rel for Ruler.fc-fill-start
                                verticalScrolling && 'fc-liquid'), ref: this.mainScrollerRef, clientWidthRef: this.handleClientWidth, clientHeightRef: this.handleClientHeight },
                                preact.createElement("div", { className: 'fc-flex-col fc-grow fc-rel', style: {
                                        width: canvasWidth,
                                        height: forcedBodyHeight,
                                    } },
                                    preact.createElement(TimeGridCols, { dateProfile: props.dateProfile, nowDate: props.nowDate, todayRange: props.todayRange, cells: props.cells, slatCnt: slatCnt, forPrint: forPrint, isHitComboAllowed: props.isHitComboAllowed, className: simplePrint ? '' : 'fc-fill', 
                                        // content
                                        fgEventSegsByCol: props.fgEventSegsByCol, bgEventSegsByCol: props.bgEventSegsByCol, businessHourSegsByCol: props.businessHourSegsByCol, nowIndicatorSegsByCol: props.nowIndicatorSegsByCol, dateSelectionSegsByCol: props.dateSelectionSegsByCol, eventDragByCol: props.eventDragByCol, eventResizeByCol: props.eventResizeByCol, eventSelection: props.eventSelection, 
                                        // dimensions
                                        colWidth: colWidth, slatHeight: slatHeight }),
                                    !simplePrint && (preact.createElement(preact.Fragment, null,
                                        preact.createElement("div", { "aria-hidden": true, className: internal$1.joinClassNames('fc-timegrid-slots fc-flex-col', (verticalScrolling && options.expandRows) && 'fc-grow', absPrint ? 'fc-fill-x' : 'fc-rel') }, props.slatMetas.map((slatMeta, slatI) => (preact.createElement("div", { key: slatMeta.key, "data-time": slatMeta.isoTimeStr, className: internal$1.joinClassNames(...getSlatRowClassNames(slatMeta), slatI && 'fc-border-t', slatLiquid && 'fc-liquid'), style: {
                                                height: slatLiquid ? '' : slatHeight
                                            } },
                                            preact.createElement(TimeGridSlatLane, Object.assign({}, slatMeta /* FYI doesn't need isoTimeStr */, { innerHeightRef: slatMainInnerHeightRefMap.createRef(slatMeta.key) })))))),
                                        rowsNotExpanding && (preact.createElement("div", { class: 'fc-liquid fc-border-t fc-filler' })))))),
                            Boolean(stickyFooterScrollbar) && (preact.createElement(internal$1.FooterScrollbar, { isSticky: true, canvasWidth: canvasWidth, scrollerRef: this.footScrollerRef, scrollbarWidthRef: this.handleBottomScrollbarWidth }))))),
                preact.createElement(internal$1.Ruler, { widthRef: this.handleTotalWidth })));
        }
        // Lifecycle
        // -----------------------------------------------------------------------------------------------
        componentDidMount() {
            this.initScrollers();
            this.updateSlatHeight();
        }
        componentDidUpdate() {
            this.updateScrollers();
            this.updateSlatHeight();
        }
        componentWillUnmount() {
            this.destroyScrollers();
            internal$1.setRef(this.props.slatHeightRef, null);
        }
        updateSlatHeight() {
            if (this.prevSlatHeight !== this.slatHeight) {
                internal$1.setRef(this.props.slatHeightRef, this.prevSlatHeight = this.slatHeight);
            }
        }
        // Scrolling
        // -----------------------------------------------------------------------------------------------
        initScrollers() {
            const ScrollerSyncer = internal$1.getScrollerSyncerClass(this.context.pluginHooks);
            this.dayScroller = new ScrollerSyncer(true); // horizontal=true
            this.timeScroller = new ScrollerSyncer(); // horizontal=false
            internal$1.setRef(this.props.dayScrollerRef, this.dayScroller);
            internal$1.setRef(this.props.timeScrollerRef, this.timeScroller);
            this.updateScrollers();
        }
        updateScrollers() {
            this.dayScroller.handleChildren([
                this.headerScrollerRef.current,
                this.allDayScrollerRef.current,
                this.mainScrollerRef.current,
                this.footScrollerRef.current,
            ]);
            this.timeScroller.handleChildren([
                this.axisScrollerRef.current,
                this.mainScrollerRef.current,
            ]);
        }
        destroyScrollers() {
            internal$1.setRef(this.props.dayScrollerRef, null);
            internal$1.setRef(this.props.timeScrollerRef, null);
        }
    }

    class TimeGridLayoutNormal extends internal$1.BaseComponent {
        constructor() {
            super(...arguments);
            // refs
            this.headerLabelInnerWidthRefMap = new internal$1.RefMap(() => {
                internal$1.afterSize(this.handleAxisInnerWidths);
            });
            this.handleAllDayLabelInnerWidth = (width) => {
                this.allDayLabelInnerWidth = width;
                internal$1.afterSize(this.handleAxisInnerWidths);
            };
            this.handleWeekNumberInnerWidth = (width) => {
                this.weekNumberInnerWidth = width;
                internal$1.afterSize(this.handleAxisInnerWidths);
            };
            this.slatLabelInnerWidthRefMap = new internal$1.RefMap(() => {
                internal$1.afterSize(this.handleAxisInnerWidths);
            });
            this.slatLabelInnerHeightRefMap = new internal$1.RefMap(() => {
                internal$1.afterSize(this.handleSlatInnerHeights);
            });
            this.slatInnerMainHeightRefMap = new internal$1.RefMap(() => {
                internal$1.afterSize(this.handleSlatInnerHeights);
            });
            // Sizing
            // -----------------------------------------------------------------------------------------------
            this.handleTotalWidth = (totalWidth) => {
                // Must delay the rerender because might change the width of the all-day DayGridRow events,
                // which shows a ResizeObserver loop warning
                requestAnimationFrame(() => {
                    this.setState({ totalWidth });
                });
            };
            this.handleClientWidth = (clientWidth) => {
                this.setState({ clientWidth });
            };
            this.handleClientHeight = (clientHeight) => {
                this.setState({ clientHeight });
            };
            this.handleAxisInnerWidths = () => {
                const headerLabelInnerWidthMap = this.headerLabelInnerWidthRefMap.current;
                const slatLabelInnerWidthMap = this.slatLabelInnerWidthRefMap.current;
                let max = Math.max(this.weekNumberInnerWidth || 0, // might not exist
                this.allDayLabelInnerWidth || 0 // guard against all-day slot hidden
                );
                for (const headerLabelInnerWidth of headerLabelInnerWidthMap.values()) {
                    max = Math.max(max, headerLabelInnerWidth);
                }
                for (const slatLabelInnerWidth of slatLabelInnerWidthMap.values()) {
                    max = Math.max(max, slatLabelInnerWidth);
                }
                if (this.state.axisWidth !== max) {
                    this.setState({ axisWidth: max });
                }
            };
            this.handleSlatInnerHeights = () => {
                const slatLabelInnerHeightMap = this.slatLabelInnerHeightRefMap.current;
                const slatMainInnerHeightMap = this.slatInnerMainHeightRefMap.current;
                let max = 0;
                for (const slatLabelInnerHeight of slatLabelInnerHeightMap.values()) {
                    max = Math.max(max, slatLabelInnerHeight);
                }
                for (const slatInnerHeight of slatMainInnerHeightMap.values()) {
                    max = Math.max(max, slatInnerHeight);
                }
                if (this.state.slatInnerHeight !== max) {
                    this.setState({ slatInnerHeight: max });
                }
            };
        }
        render() {
            const { props, state, context, slatLabelInnerWidthRefMap, slatLabelInnerHeightRefMap, slatInnerMainHeightRefMap, headerLabelInnerWidthRefMap } = this;
            const { nowDate, forPrint } = props;
            const { axisWidth, clientWidth, totalWidth } = state;
            const { options } = context;
            const endScrollbarWidth = (totalWidth != null && clientWidth != null)
                ? totalWidth - clientWidth
                : undefined;
            const verticalScrolling = !forPrint && !internal$1.getIsHeightAuto(options);
            const stickyHeaderDates = !forPrint && internal$1.getStickyHeaderDates(options);
            const slatCnt = props.slatMetas.length;
            const [slatHeight, slatLiquid] = computeSlatHeight(verticalScrolling && options.expandRows, slatCnt, state.slatInnerHeight, state.clientHeight);
            this.slatHeight = slatHeight;
            // TODO: have computeSlatHeight return?
            const totalSlatHeight = (slatHeight || 0) * slatCnt;
            const rowsNotExpanding = verticalScrolling && !options.expandRows &&
                state.clientHeight != null && state.clientHeight > totalSlatHeight;
            const absPrint = forPrint && !simplifiedTimeGridPrint;
            const simplePrint = forPrint && simplifiedTimeGridPrint;
            // for printing
            // in Chrome, slats and columns both need abs positioning within a relative container for them
            // to sync across pages, and the relative container needs an explicit height
            // in Firefox, same applies, but the flex-row for the cells has trouble spanning across page,
            // so we need to set explicit height on flex-row and all parents
            const forcedBodyHeight = absPrint ? totalSlatHeight : undefined;
            return (preact.createElement(preact.Fragment, null,
                options.dayHeaders && (preact.createElement("div", { role: 'rowgroup', className: internal$1.joinClassNames(
                    // see note in TimeGridLayout about why we don't do fc-print-header
                    'fc-timegrid-header fc-flex-col fc-border-b', stickyHeaderDates && 'fc-table-header-sticky') }, props.headerTiers.map((rowConfig, tierNum) => (preact.createElement("div", { key: tierNum, role: 'row', className: internal$1.joinClassNames('fc-flex-row', tierNum && 'fc-border-t') },
                    (options.weekNumbers && rowConfig.isDateRow) ? (preact.createElement(TimeGridWeekNumber, { dateProfile: props.dateProfile, innerWidthRef: this.handleWeekNumberInnerWidth, innerHeightRef: headerLabelInnerWidthRefMap.createRef(tierNum), width: axisWidth, isLiquid: false })) : (preact.createElement(TimeGridAxisEmpty, { width: axisWidth, isLiquid: false })),
                    preact.createElement(internal$2.DayGridHeaderRow, Object.assign({}, rowConfig, { className: 'fc-border-s fc-liquid' })),
                    Boolean(endScrollbarWidth) && (preact.createElement("div", { className: 'fc-border-s fc-filler', style: { minWidth: endScrollbarWidth } }))))))),
                preact.createElement("div", { role: 'rowgroup', className: internal$1.joinClassNames('fc-flex-col', verticalScrolling && 'fc-liquid') },
                    options.allDaySlot && (preact.createElement(preact.Fragment, null,
                        preact.createElement("div", { role: 'row', className: 'fc-timegrid-allday fc-flex-row' },
                            preact.createElement(TimeGridAllDayLabel, { width: axisWidth, innerWidthRef: this.handleAllDayLabelInnerWidth }),
                            preact.createElement(TimeGridAllDayLane, { dateProfile: props.dateProfile, todayRange: props.todayRange, cells: props.cells, showDayNumbers: false, forPrint: forPrint, isHitComboAllowed: props.isHitComboAllowed, className: 'fc-border-s fc-liquid', isCompact: internal$2.computeRowIsCompact(clientWidth, options), 
                                // content
                                fgEventSegs: props.fgEventSegs, bgEventSegs: props.bgEventSegs, businessHourSegs: props.businessHourSegs, dateSelectionSegs: props.dateSelectionSegs, eventDrag: props.eventDrag, eventResize: props.eventResize, eventSelection: props.eventSelection, dayMaxEvents: props.dayMaxEvents, dayMaxEventRows: props.dayMaxEventRows }),
                            Boolean(endScrollbarWidth) && (preact.createElement("div", { className: 'fc-border-s fc-filler', style: { minWidth: endScrollbarWidth } }))),
                        preact.createElement("div", { className: 'fc-rowdivider' }))),
                    preact.createElement(internal$1.Scroller, { vertical: verticalScrolling, className: internal$1.joinClassNames('fc-timegrid-body fc-flex-col fc-rel', // fc-rel for Ruler.fc-fill-start
                        verticalScrolling && 'fc-liquid'), ref: props.timeScrollerRef, clientWidthRef: this.handleClientWidth, clientHeightRef: this.handleClientHeight },
                        preact.createElement("div", { className: 'fc-flex-col fc-grow fc-rel', style: {
                                // in print mode, this div creates the height and everything is absolutely positioned within
                                // we need to do this so that slats positioning synces with events's positioning
                                // otherwise, get out of sync on second page
                                height: forcedBodyHeight,
                            } },
                            preact.createElement("div", { role: 'row', className: internal$1.joinClassNames('fc-flex-row', !simplePrint && 'fc-fill') },
                                preact.createElement("div", { role: 'rowheader', "aria-label": options.timedText, className: 'fc-content-box', style: { width: axisWidth } }),
                                preact.createElement(TimeGridCols, { dateProfile: props.dateProfile, nowDate: props.nowDate, todayRange: props.todayRange, cells: props.cells, slatCnt: slatCnt, forPrint: forPrint, isHitComboAllowed: props.isHitComboAllowed, className: 'fc-liquid fc-border-s', 
                                    // content
                                    fgEventSegsByCol: props.fgEventSegsByCol, bgEventSegsByCol: props.bgEventSegsByCol, businessHourSegsByCol: props.businessHourSegsByCol, nowIndicatorSegsByCol: props.nowIndicatorSegsByCol, dateSelectionSegsByCol: props.dateSelectionSegsByCol, eventDragByCol: props.eventDragByCol, eventResizeByCol: props.eventResizeByCol, eventSelection: props.eventSelection, 
                                    // dimensions
                                    slatHeight: slatHeight })),
                            !simplePrint && (preact.createElement(preact.Fragment, null,
                                preact.createElement("div", { "aria-hidden": true, className: internal$1.joinClassNames('fc-timegrid-slots fc-flex-col', (verticalScrolling && options.expandRows) && 'fc-grow', absPrint
                                        ? 'fc-fill-x' // will assume top:0, height will be decided naturally
                                        : 'fc-rel') }, props.slatMetas.map((slatMeta, slatI) => (preact.createElement("div", { key: slatMeta.key, "data-time": slatMeta.isoTimeStr, className: internal$1.joinClassNames(...getSlatRowClassNames(slatMeta), slatI && 'fc-border-t', slatLiquid && 'fc-liquid'), style: {
                                        height: slatLiquid ? '' : slatHeight
                                    } },
                                    preact.createElement(TimeGridSlatLabel, Object.assign({}, slatMeta /* FYI doesn't need isoTimeStr */, { innerWidthRef: slatLabelInnerWidthRefMap.createRef(slatMeta.key), innerHeightRef: slatLabelInnerHeightRefMap.createRef(slatMeta.key), width: axisWidth })),
                                    preact.createElement(TimeGridSlatLane, Object.assign({}, slatMeta /* FYI doesn't need isoTimeStr */, { borderStart: true, innerHeightRef: slatInnerMainHeightRefMap.createRef(slatMeta.key) })))))),
                                rowsNotExpanding && (preact.createElement("div", { class: 'fc-liquid fc-border-t fc-filler' })),
                                options.nowIndicator && internal$1.rangeContainsMarker(props.dateProfile.currentRange, nowDate) && (preact.createElement(TimeGridNowIndicatorArrow, { nowDate: nowDate, dateProfile: props.dateProfile, totalHeight: slatHeight != null ? slatHeight * slatCnt : undefined }))))))),
                preact.createElement(internal$1.Ruler, { widthRef: this.handleTotalWidth })));
        }
        // Lifecycle
        // -----------------------------------------------------------------------------------------------
        componentDidMount() {
            this.updateSlatHeight();
        }
        componentDidUpdate() {
            this.updateSlatHeight();
        }
        componentWillUnmount() {
            internal$1.setRef(this.props.slatHeightRef, null);
        }
        updateSlatHeight() {
            if (this.prevSlatHeight !== this.slatHeight) {
                internal$1.setRef(this.props.slatHeightRef, this.prevSlatHeight = this.slatHeight);
            }
        }
    }

    class TimeGridLayout extends internal$1.BaseComponent {
        constructor() {
            super(...arguments);
            // memo
            this.buildSlatMetas = internal$1.memoize(buildSlatMetas);
            // refs
            this.dayScrollerRef = preact.createRef();
            this.timeScrollerRef = preact.createRef();
            this.scrollState = {}; // updated in-place
            // Sizing
            // -----------------------------------------------------------------------------------------------
            this.handleSlatHeight = (slatHeight) => {
                this.slatHeight = slatHeight;
                if (slatHeight != null) {
                    internal$1.afterSize(this.applyTimeScroll);
                }
            };
            this.handleTimeScrollRequest = (scrollTime) => {
                this.scrollState.time = scrollTime;
                this.scrollState.y = undefined;
                this.applyTimeScroll();
            };
            /*
            Captures current values
            */
            this.handleTimeScrollEnd = (isUser) => {
                if (isUser) {
                    const y = this.timeScrollerRef.current.y;
                    // record, but only if not forPrint, which could give bogus values in the case of
                    // TimeGridLayoutPannable, which kills y-scrolling, but retains x-scrolling,
                    // which reports as a 0 y-scroll.
                    if (!this.props.forPrint) {
                        this.scrollState.y = y;
                        this.scrollState.time = undefined;
                    }
                }
            };
            this.applyTimeScroll = () => {
                const timeScroller = this.timeScrollerRef.current;
                const { slatHeight, scrollState } = this;
                let { y, time } = scrollState;
                if (y == null &&
                    time &&
                    slatHeight != null &&
                    // Since applyTimeScroll is called by handleSlatHeight, could be called with null during cleanup,
                    // and the timeScroller might not exist
                    timeScroller) {
                    y = computeTimeTopFrac(time, this.props.dateProfile)
                        * (slatHeight * this.currentSlatCnt);
                    if (y) {
                        y++; // overcome top border
                    }
                    scrollState.y = y; // HACK: store raw pixel value
                }
                if (y != null) {
                    timeScroller.scrollTo({ y });
                }
            };
        }
        render() {
            const { props, context } = this;
            const { dateProfile } = props;
            const { options, dateEnv } = context;
            const { dayMinWidth } = options;
            const slatMetas = this.buildSlatMetas(dateProfile.slotMinTime, dateProfile.slotMaxTime, options.slotLabelInterval, options.slotDuration, dateEnv);
            this.currentSlatCnt = slatMetas.length;
            const commonLayoutProps = Object.assign(Object.assign({ dateProfile: dateProfile, nowDate: props.nowDate, todayRange: props.todayRange, cells: props.cells, slatMetas, forPrint: props.forPrint, isHitComboAllowed: props.isHitComboAllowed, 
                // header content
                headerTiers: props.headerTiers, 
                // all-day content
                fgEventSegs: props.fgEventSegs, bgEventSegs: props.bgEventSegs, businessHourSegs: props.businessHourSegs, dateSelectionSegs: props.dateSelectionSegs, eventDrag: props.eventDrag, eventResize: props.eventResize }, getAllDayMaxEventProps(options)), { 
                // timed content
                fgEventSegsByCol: props.fgEventSegsByCol, bgEventSegsByCol: props.bgEventSegsByCol, businessHourSegsByCol: props.businessHourSegsByCol, nowIndicatorSegsByCol: props.nowIndicatorSegsByCol, dateSelectionSegsByCol: props.dateSelectionSegsByCol, eventDragByCol: props.eventDragByCol, eventResizeByCol: props.eventResizeByCol, 
                // universal content
                eventSelection: props.eventSelection, 
                // refs
                timeScrollerRef: this.timeScrollerRef, timeScrollState: this.scrollState, slatHeightRef: this.handleSlatHeight });
            return (preact.createElement(internal$1.ViewContainer, { attrs: {
                    role: 'grid',
                    'aria-colcount': props.cells.length,
                    'aria-labelledby': props.labelId,
                    'aria-label': props.labelStr,
                }, className: internal$1.joinClassNames(props.className, 'fc-border', !props.forPrint && 'fc-flex-col'), viewSpec: context.viewSpec }, dayMinWidth ? (preact.createElement(TimeGridLayoutPannable, Object.assign({}, commonLayoutProps, { dayMinWidth: dayMinWidth, dayScrollerRef: this.dayScrollerRef }))) : (preact.createElement(TimeGridLayoutNormal, Object.assign({}, commonLayoutProps)))));
        }
        // Lifecycle
        // -----------------------------------------------------------------------------------------------
        componentDidMount() {
            this.resetScroll();
            this.context.emitter.on('_timeScrollRequest', this.handleTimeScrollRequest);
            this.timeScrollerRef.current.addScrollEndListener(this.handleTimeScrollEnd);
        }
        componentDidUpdate(prevProps) {
            if (prevProps.dateProfile !== this.props.dateProfile && this.context.options.scrollTimeReset) {
                this.resetScroll();
            }
            else if (prevProps.forPrint && !this.props.forPrint) {
                // returning from print
                // reapply scrolling because scroll-divs were probably restored
                this.applyTimeScroll();
            }
        }
        componentWillUnmount() {
            this.context.emitter.off('_timeScrollRequest', this.handleTimeScrollRequest);
            this.timeScrollerRef.current.removeScrollEndListener(this.handleTimeScrollEnd);
        }
        // Scrolling
        // -----------------------------------------------------------------------------------------------
        resetScroll() {
            this.handleTimeScrollRequest(this.context.options.scrollTime);
            // also resets day scroll
            const dayScroller = this.dayScrollerRef.current;
            if (dayScroller) {
                dayScroller.scrollTo({ x: 0 });
            }
        }
    }
    // Utils
    // -----------------------------------------------------------------------------------------------
    const AUTO_ALL_DAY_MAX_EVENT_ROWS = 5;
    function getAllDayMaxEventProps(options) {
        let { dayMaxEvents, dayMaxEventRows } = options;
        if (dayMaxEvents === true || dayMaxEventRows === true) { // is auto?
            dayMaxEvents = undefined;
            dayMaxEventRows = AUTO_ALL_DAY_MAX_EVENT_ROWS; // make sure "auto" goes to a real number
        }
        return { dayMaxEvents, dayMaxEventRows };
    }

    class TimeGridView extends internal$1.DateComponent {
        constructor() {
            super(...arguments);
            // memo
            this.createDayHeaderFormatter = internal$1.memoize(internal$2.createDayHeaderFormatter);
            this.buildTimeColsModel = internal$1.memoize(buildTimeColsModel);
            this.buildDayRanges = internal$1.memoize(buildDayRanges);
            this.buildDateRowConfigs = internal$1.memoize(internal$2.buildDateRowConfigs);
            this.splitFgEventSegs = internal$1.memoize((organizeSegsByCol));
            this.splitBgEventSegs = internal$1.memoize((organizeSegsByCol));
            this.splitBusinessHourSegs = internal$1.memoize((organizeSegsByCol));
            this.splitNowIndicatorSegs = internal$1.memoize((organizeSegsByCol));
            this.splitDateSelectionSegs = internal$1.memoize((organizeSegsByCol));
            this.splitEventDrag = internal$1.memoize(splitInteractionByCol);
            this.splitEventResize = internal$1.memoize(splitInteractionByCol);
            // internal
            this.allDaySplitter = new AllDaySplitter();
            this.dayTableSlicer = new internal$2.DayTableSlicer();
            this.dayTimeColsSlicer = new DayTimeColsSlicer();
        }
        render() {
            const { props, context } = this;
            const { dateProfile } = props;
            const { options, dateProfileGenerator } = context;
            const dayTableModel = this.buildTimeColsModel(dateProfile, dateProfileGenerator);
            const dayRanges = this.buildDayRanges(dayTableModel, dateProfile, context.dateEnv);
            const splitProps = this.allDaySplitter.splitProps(props);
            const allDayProps = this.dayTableSlicer.sliceProps(splitProps.allDay, dateProfile, options.nextDayThreshold, context, dayTableModel);
            const timedProps = this.dayTimeColsSlicer.sliceProps(splitProps.timed, dateProfile, null, context, dayRanges);
            const dayHeaderFormat = this.createDayHeaderFormatter(context.options.dayHeaderFormat, true, // datesRepDistinctDays
            dayTableModel.colCnt);
            return (preact.createElement(internal$1.NowTimer, { unit: options.nowIndicator ? 'minute' : 'day' /* hacky */ }, (nowDate, todayRange) => {
                const colCnt = dayTableModel.cellRows[0].length;
                const nowIndicatorSeg = options.nowIndicator &&
                    this.dayTimeColsSlicer.sliceNowDate(nowDate, dateProfile, options.nextDayThreshold, context, dayRanges);
                const fgEventSegsByCol = this.splitFgEventSegs(timedProps.fgEventSegs, colCnt);
                const bgEventSegsByCol = this.splitBgEventSegs(timedProps.bgEventSegs, colCnt);
                const businessHourSegsByCol = this.splitBusinessHourSegs(timedProps.businessHourSegs, colCnt);
                const nowIndicatorSegsByCol = this.splitNowIndicatorSegs(nowIndicatorSeg, colCnt);
                const dateSelectionSegsByCol = this.splitDateSelectionSegs(timedProps.dateSelectionSegs, colCnt);
                const eventDragByCol = this.splitEventDrag(timedProps.eventDrag, colCnt);
                const eventResizeByCol = this.splitEventResize(timedProps.eventResize, colCnt);
                const headerTiers = this.buildDateRowConfigs(dayTableModel.headerDates, true, // datesRepDistinctDays
                props.dateProfile, todayRange, dayHeaderFormat, context);
                return (preact.createElement(TimeGridLayout, { labelId: props.labelId, labelStr: props.labelStr, dateProfile: dateProfile, nowDate: nowDate, todayRange: todayRange, cells: dayTableModel.cellRows[0], forPrint: props.forPrint, className: 'fc-timegrid', 
                    // header content
                    headerTiers: headerTiers, 
                    // all-day content
                    fgEventSegs: allDayProps.fgEventSegs, bgEventSegs: allDayProps.bgEventSegs, businessHourSegs: allDayProps.businessHourSegs, dateSelectionSegs: allDayProps.dateSelectionSegs, eventDrag: allDayProps.eventDrag, eventResize: allDayProps.eventResize, 
                    // timed content
                    fgEventSegsByCol: fgEventSegsByCol, bgEventSegsByCol: bgEventSegsByCol, businessHourSegsByCol: businessHourSegsByCol, nowIndicatorSegsByCol: nowIndicatorSegsByCol, dateSelectionSegsByCol: dateSelectionSegsByCol, eventDragByCol: eventDragByCol, eventResizeByCol: eventResizeByCol, 
                    // universal content
                    eventSelection: props.eventSelection }));
            }));
        }
    }

    const OPTION_REFINERS = {
        allDaySlot: Boolean,
    };

    var css_248z = ".fc-v-event{background-color:var(--fc-event-bg-color);border:1px solid var(--fc-event-border-color);display:flex;flex-direction:column}.fc-v-event.fc-event-mirror{z-index:1}.fc-v-event:not(.fc-event-start){border-top-left-radius:0;border-top-right-radius:0;border-top-width:0}.fc-v-event:not(.fc-event-end){border-bottom-left-radius:0;border-bottom-right-radius:0;border-bottom-width:0}.fc-v-event.fc-event-selected:before{left:-10px;right:-10px}.fc-v-event .fc-event-inner{color:var(--fc-event-text-color);display:flex;flex-direction:column;flex-grow:1;min-height:0}.fc-v-event .fc-event-time{flex-shrink:0;white-space:nowrap}.fc-v-event .fc-event-time,.fc-v-event .fc-event-title{overflow:hidden}.fc-v-event .fc-event-title-outer{display:flex;flex-direction:column;flex-grow:1;min-height:0}.fc-v-event .fc-event-title{bottom:0;position:sticky;top:0}.fc-v-event .fc-event-resizer-start{cursor:n-resize}.fc-v-event .fc-event-resizer-end{cursor:s-resize}.fc-v-event:not(.fc-event-selected) .fc-event-resizer{height:var(--fc-event-resizer-thickness);left:0;right:0}.fc-v-event:not(.fc-event-selected) .fc-event-resizer-start{top:calc(var(--fc-event-resizer-thickness)/-2)}.fc-v-event:not(.fc-event-selected) .fc-event-resizer-end{bottom:calc(var(--fc-event-resizer-thickness)/-2)}.fc-v-event.fc-event-selected .fc-event-resizer{left:50%;margin-left:calc(var(--fc-event-resizer-dot-total-width)/-2)}.fc-v-event.fc-event-selected .fc-event-resizer-start{top:calc(var(--fc-event-resizer-dot-total-width)/-2)}.fc-v-event.fc-event-selected .fc-event-resizer-end{bottom:calc(var(--fc-event-resizer-dot-total-width)/-2)}.fc-timegrid-slot-minor{border-style:dotted}.fc-timegrid-axis{align-items:flex-end}.fc-timegrid-axis,.fc-timegrid-axis-inner{display:flex;flex-direction:column;justify-content:center}.fc-timegrid-axis-inner{min-height:1.5em}.fc-direction-ltr .fc-timegrid-axis-inner{text-align:right}.fc-direction-rtl .fc-timegrid-axis-inner{text-align:left}.fc-timegrid-allday-label-inner{white-space:pre}.fc-timegrid-day.fc-day-today{background-color:var(--fc-today-bg-color)}.fc-direction-ltr .fc-timegrid-day-events{margin:0 2.5% 0 2px}.fc-direction-rtl .fc-timegrid-day-events{margin:0 2px 0 2.5%}.fc-timegrid-day-events.fc-timegrid-day-events-simple{margin:4px}.fc-timegrid-event{border-radius:3px;font-size:var(--fc-small-font-size);position:relative}.fc-timegrid-event-inset,.fc-timegrid-event.fc-event-mirror{box-shadow:0 0 0 1px var(--fc-page-bg-color)}.fc-timegrid-event.fc-event-end{margin-bottom:1px}.fc-timegrid-event .fc-event-inner{padding:1px 1px 0}.fc-timegrid-event-short .fc-event-inner{flex-direction:row;overflow:hidden}.fc-timegrid-event .fc-event-time{font-size:var(--fc-small-font-size);margin-bottom:1px;white-space:nowrap}.fc-timegrid-event-short .fc-event-time:after{content:\"\\00a0-\\00a0\"}.fc-timegrid-event-short .fc-event-title-outer{flex-basis:0;flex-grow:1;min-height:0;min-width:0}.fc-timegrid-event-short .fc-event-title,.fc-timegrid-more-link{font-size:var(--fc-small-font-size)}.fc-timegrid-more-link{background:var(--fc-more-link-bg-color);border-radius:3px;box-shadow:0 0 0 1px var(--fc-page-bg-color);color:var(--fc-more-link-text-color);cursor:pointer;margin-bottom:1px;z-index:9999}.fc-direction-ltr .fc-timegrid-more-link{right:0}.fc-direction-rtl .fc-timegrid-more-link{left:0}.fc-timegrid-more-link-inner{padding:3px 2px}.fc-timegrid-now-indicator-container{bottom:0;left:0;overflow:hidden;pointer-events:none;position:absolute;right:0;top:0;z-index:2}.fc-timegrid-now-indicator-arrow{border-color:var(--fc-now-indicator-color);border-bottom:5px solid transparent;border-left-style:solid;border-left-width:6px;border-right-style:solid;border-right-width:6px;border-top:5px solid transparent;height:0;margin-top:-5px;position:absolute;width:0}.fc-direction-ltr .fc-timegrid-now-indicator-arrow{border-right:0;left:0}.fc-direction-rtl .fc-timegrid-now-indicator-arrow{border-left:0;right:0}.fc-timegrid-now-indicator-line{border-top:1px solid var(--fc-now-indicator-color);left:0;position:absolute;right:0}";
    internal$1.injectStyles(css_248z);

    var plugin = core.createPlugin({
        name: '@fullcalendar/timegrid',
        initialView: 'timeGridWeek',
        optionRefiners: OPTION_REFINERS,
        views: {
            timeGrid: {
                component: TimeGridView,
                usesMinMaxTime: true,
                allDaySlot: true,
                slotDuration: '00:30:00',
                slotEventOverlap: true, // a bad name. confused with overlap/constraint system
            },
            timeGridDay: {
                type: 'timeGrid',
                duration: { days: 1 },
            },
            timeGridWeek: {
                type: 'timeGrid',
                duration: { weeks: 1 },
            },
        },
    });

    var internal = {
        __proto__: null,
        organizeSegsByCol: organizeSegsByCol,
        splitInteractionByCol: splitInteractionByCol,
        DayTimeColsSlicer: DayTimeColsSlicer,
        buildSlatMetas: buildSlatMetas,
        AllDaySplitter: AllDaySplitter,
        TimeGridView: TimeGridView,
        TimeGridLayout: TimeGridLayout,
        TimeGridWeekNumber: TimeGridWeekNumber,
        buildTimeColsModel: buildTimeColsModel,
        buildDayRanges: buildDayRanges
    };

    core.globalPlugins.push(plugin);

    exports.Internal = internal;
    exports["default"] = plugin;

    Object.defineProperty(exports, '__esModule', { value: true });

    return exports;

})({}, FullCalendar, FullCalendar.Internal, FullCalendar.Preact, FullCalendar.DayGrid.Internal);
